function [w, bias] = cost_sensitive_micro(train_data, options)
    % This method select a parameter -w1 for all labels to optimize micro-F.
    % input:
    %   train_data: a struct with `y`,`x`,and `map` return by libsvmread_ml
    %   options: options to pass to LIBLINEAR
    % output:
    %   w: weight of the linear SVMs. The shape is (#dim, #label)
    %   bias: the -B value passed to LIBLINEAR. -1 if not passed

    % check input
    if (nargin ~= 2)
        if nargin < 2
            options = '-q -B 1';
        end
        if nargin < 1 || nargin > 2
            fprintf('Incorrect number of input parameters\n');
            return;
        end
    end
    
    x = train_data.x;
    y = train_data.y;
    map = train_data.map;

    % bias option
    bias = get_bias(options);
    if bias > 0
        w_size = size(x, 1) + 1;
    else
        w_size = size(x, 1);
    end

    t0all = clock;
    
    % number of label = number of model to be trained
    L = size(y, 2); 
    
    param_space = [1, 1.33, 1.8, 2.5, 3.67, 6, 13];
    best_param = 1;
    best_F = -1;

    for param = param_space
        counts = zeros(3,L); % tp,fp,fn
        parfor i = 1:L
            fprintf(1, 'INFO: cross-validating %dth label (%d)...', i, map(i));
            t0 = clock;
            counts(:,i) = cv_one(y(:, i)*2 - 1, x, [options ' -w1 ' num2str(param)]);
            fprintf(1, '\tDone in %.6f seconds\n', etime(clock, t0));
        end
        counts = sum(counts,2);
        F = 2*counts(1) / (2*counts(1) + counts(2) + counts(3))
        if best_F < F
            best_F = F;
            best_param = param;
        end
    end

    w = zeros(w_size, L);
    parfor i = 1:L
        fprintf(1, 'INFO: training %dth label (%d)...', i, map(i));
        t0 = clock;
        [w(:, i)] = do_train(y(:, i)*2 - 1, x, [options ' -w1 ' num2str(best_param)]);
        fprintf(1, '\tDone in %.6f seconds\n', etime(clock, t0));
    end

    fprintf(1, 'INFO: Total Time: %.6f seconds\n', etime(clock, t0all));
end

function counts = cv_one(proby, probx, options)
    % perform CV and return the tp,fp,fn
    % inputs:
    %   proby: vector of size (#sample,1), elements are -1,+1
    %   probx: feature matrix of size (#feature, #sample)
    %   options: options to be pass to LIBLINEAR

    nr_fold = 3;

    bias = get_bias(options);

    warning('off', 'stats:cvpartition:KFoldMissingGrp');
    cv = cvpartition(proby, 'KFold', nr_fold, 'Stratify', true);

    % cv
    pred = zeros(numel(proby),1);
    for fold = 1:nr_fold
        train_id = cv.training(fold);
        valid_id = cv.test(fold);

        train_y = proby(train_id);
        train_x = probx(:, train_id);
        valid_x = probx(:, valid_id);

        % scut
        scut_w = do_train(train_y, train_x, options);
        if bias > 0
            valid_x = [valid_x; ones(1, size(valid_x, 2)) * bias];
        end
        pred(valid_id) = valid_x' * scut_w; % shape = (#sample in valid)
    end
    pred = 2*(pred > 0) - 1;

    tp = sum(proby == +1 & pred == +1);
    fp = sum(proby == -1 & pred == +1);
    fn = sum(proby == +1 & pred == -1);

    counts = [tp,fp,fn];
end

