function [w, T, bias] = micromacro_freq(train_data, options, train_final)
    % micromacro (jointly optimize micro-F and macro-F)
    % Frequent labels first version !
    % input:
    %   train_data: a struct with `y`,`x`,and `map` return by libsvmread_ml
    %   options: options to pass to LIBLINEAR
    %   train_final: whether to train the final linear model using all training data
    % output:
    %   w: weight of the linear SVMs. The shape is (#dim, #label)
    %   T: threshold for each label. The shape is (#label, 1)
    %   bias: the -B value passed to LIBLINEAR. -1 if not passed
    

    % check input
    if (nargin ~= 3)
        if nargin < 3
            train_final = true;
        end
        if nargin < 2
            options = '-q -B 1';
        end
        if nargin < 1 || nargin > 3
            fprintf('Incorrect number of input parameters\n');
            return;
        end
    end
    
    x = train_data.x;
    y = train_data.y;
    map = train_data.map;

    % bias option
    bias = get_bias(options);
    if bias > 0
        w_size = size(x, 1) + 1;
    else
        w_size = size(x, 1);
    end

    t0all = clock;
    
    % number of label = number of model to be trained
    L = size(y, 2); 
    
    w = zeros(w_size, L);
    T = zeros(L, 1);

    [~, perm] = sort(sum(y, 1), 'descend');
    
    % parpool(4);
    % accumulated count for micro
    tp = 0; fp = 0; fn = 0;
    for j = 1:L % obtain (w,T) for each label
        i = perm(j);
        fprintf(1, 'INFO: training %dth label (%d)...', i, map(i));
        t0 = clock;
        % do not calculate y*2 - 1 at once
        [w(:, i), T(i), tp, fp, fn] = micromacro_one(y(:, i)*2 - 1, x, options, tp, fp, fn, i, train_final);
        % fprintf(1, 'INFO: b = %g\n', b(i));
        fprintf(1, '\tDone in %.6f seconds\n', etime(clock, t0));
    end
    fprintf(1, 'INFO: Total Time: %.6f seconds\n', etime(clock, t0all));
end

function [w, T, new_tp, new_fp, new_fn] = micromacro_one(...
    proby, probx, options, acc_tp, acc_fp, acc_fn, j, train_final)
    % perform CV to select the best threshold in terms of micro + macro
    % inputs:
    %   proby: vector of size (#sample,1), elements are -1,+1
    %   probx: feature matrix of size (#feature, #sample)
    %   options: options to be pass to LIBLINEAR
    %   acc_tp, acc_fp, acc_fn: accumulated tp,fp and fn that is used to calculate micro
    %   j: the current number of binary models considered. 1/j is used to scale macro
    %   train_final: whether to train the final linear model using all data

    nr_fold = 3;
    Ts = zeros(nr_fold, 1);
    
    bias = get_bias(options);

    warning('off', 'stats:cvpartition:KFoldMissingGrp');
    cv = cvpartition(proby, 'KFold', nr_fold, 'Stratify', true);

    new_tp = acc_tp;
    new_fp = acc_fp;
    new_fn = acc_fn;

    % cv
    for fold = 1:nr_fold
        train_id = cv.training(fold);
        valid_id = cv.test(fold);

        train_y = proby(train_id);
        train_x = probx(:, train_id);
        valid_y = proby(valid_id);
        valid_x = probx(:, valid_id);

        % scut
        scut_w = do_train(train_y, train_x, options);
        if bias > 0
            valid_x = [valid_x; ones(1, size(valid_x, 2)) * bias];
        end
        wTx = valid_x' *  scut_w; % shape = (#sample in valid)
        T = 0;

        start_macro = fmeasure(valid_y, 2*(wTx > T)-1);

        [sorted_wTx, sorted_wTx_index] = sort(wTx, 1, 'ascend'); 

        % best_F is initialized as predicting all as 1,
        % that is, the threshhold is set to the lowest
        tp = sum(valid_y == 1); fp = size(valid_y,1) - tp; fn = 0;
        cut = -1;
        macro = (2*tp) / (2*tp + fp + fn);
        micro = (2*(tp+acc_tp)) / (2*(tp+acc_tp) + fp + fn + acc_fp + acc_fn);
        best_F = micro + macro / j;
        best_tp = tp; best_fp = fp; best_fn = fn;

        for i = 1:size(valid_y, 1)
            if valid_y(sorted_wTx_index(i)) == -1,
                fp = fp - 1;
            else
                tp = tp - 1; fn = fn + 1;
            end
            macro = (2*tp) / (2*tp + fp + fn);
            micro = (2*(tp+acc_tp)) / (2*(tp+acc_tp) + fp + fn + acc_fp + acc_fn);
            F = micro + macro / j;

            if F >= best_F
                best_F = F;
                cut = i;
                best_tp = tp;
                best_fp = fp;
                best_fn = fn;
            end
        end

        % modify b
        if cut == -1 % i.e., all +1
            val = sorted_wTx(1);
            T = (val - eps(val)); % predict all +1
        elseif cut == size(valid_y, 1)
            val = sorted_wTx(size(valid_y, 1));
            T = (val + eps(val));
        else
            T = (sorted_wTx(cut) + sorted_wTx(cut+1))/2;
        end

        Ts(fold) = T;
        new_tp = new_tp + best_tp;
        new_fn = new_fn + best_fn;
        new_fp = new_fp + best_fp;
    end

    % final model
    T = mean(Ts);
    if train_final
        w = do_train(proby, probx, options);
    else
        if bias > 0
            w_size = size(probx, 1) + 1;
        else
            w_size = size(probx, 1);
        end
        w = zeros(w_size, 1);
    end
end

